const int mosfetPin1 = 9;    // MOSFET for Motor 1
const int encA1 = 2;         // Motor 1 Encoder A (interrupt)
const int encB1 = 11;        // Motor 1 Encoder B (direction)

const int mosfetPin2 = 10;   // MOSFET for Motor 2
const int encA2 = 3;         // Motor 2 Encoder A (interrupt)
const int encB2 = 12;        // Motor 2 Encoder B (direction)

volatile int pulseCount1 = 0; 
volatile int pulseCount2 = 0; 
const int CPR = 48; // Counts per revolution (update if needed)

unsigned long lastTime = 0;
float rpm1 = 0;
float rpm2 = 0;

void countPulses1() {  // Encoder interrupt for Motor 1
  if (digitalRead(encA1) == digitalRead(encB1)) {  
    pulseCount1++;  // One direction  
  } else {  
    pulseCount1--;  // Opposite direction  
  }  
}

void countPulses2() {  // Encoder interrupt for Motor 2
  if (digitalRead(encA2) == digitalRead(encB2)) {  
    pulseCount2++;  // One direction  
  } else {  
    pulseCount2--;  // Opposite direction  
  }  
}

void setup() {
  pinMode(mosfetPin1, OUTPUT);    // Initialize Motor 1 MOSFET pin
  pinMode(encA1, INPUT_PULLUP);   // Motor 1 Encoder A pin
  pinMode(encB1, INPUT_PULLUP);   // Motor 1 Encoder B pin

  pinMode(mosfetPin2, OUTPUT);    // Initialize Motor 2 MOSFET pin
  pinMode(encA2, INPUT_PULLUP);   // Motor 2 Encoder A pin
  pinMode(encB2, INPUT_PULLUP);   // Motor 2 Encoder B pin

  attachInterrupt(digitalPinToInterrupt(encA1), countPulses1, RISING);  // Interrupt for Motor 1 Encoder
  attachInterrupt(digitalPinToInterrupt(encA2), countPulses2, RISING);  // Interrupt for Motor 2 Encoder

  Serial.begin(115200);  // Start serial communication

  analogWrite(mosfetPin1, 255);  // Run motor 1 indefinitely at full speed
  analogWrite(mosfetPin2, 255);  // Run motor 2 indefinitely at full speed
}

void loop() {
  unsigned long currentTime = millis();
  
  // Calculate RPM every 1 second
  if (currentTime - lastTime >= 1000) {
    rpm1 = (abs(pulseCount1) * 60.0) / (CPR * 2);
    rpm2 = (abs(pulseCount2) * 60.0) / (CPR * 2);

    Serial.print("Motor 1 RPM: ");
    Serial.println(rpm1);
    Serial.print("Motor 2 RPM: ");
    Serial.println(rpm2);

    pulseCount1 = 0; // Reset pulse count for Motor 1
    pulseCount2 = 0; // Reset pulse count for Motor 2
    lastTime = currentTime;
  }

  delay(50);  // Small delay for stability
}
